#!/usr/bin/env python3
import serial
import time
import sys

# Liste gängiger Baudraten
BAUDRATES = [1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200, 230400, 460800, 500000, 576000, 921600, 1000000, 1152000, 1500000, 2000000, 2500000, 3000000, 3500000, 4000000]

# Gerätepfad
PORT = "/dev/ttyUSB1"  # Anpassen falls nötig

def test_baudrate(port, baudrate):
    print(f"Teste Baudrate: {baudrate} bps...")
    try:
        # Serielle Verbindung öffnen
        ser = serial.Serial(
            port=port,
            baudrate=baudrate,
            bytesize=serial.EIGHTBITS,
            parity=serial.PARITY_NONE,
            stopbits=serial.STOPBITS_ONE,
            timeout=1
        )
        
        # Puffer leeren
        ser.reset_input_buffer()
        
        # Daten sammeln
        received_data = []
        start_time = time.time()
        
        # 3 Sekunden lang Daten sammeln
        while time.time() - start_time < 3:
            if ser.in_waiting > 0:
                data = ser.read(ser.in_waiting)
                received_data.append(data)
                
                # Zeige die empfangenen Daten an
                print(f"  Empfangen: {data}")
                
                # Prüfe auf das 'U' Zeichen
                if b'U' in data:
                    print(f"\nERFOLG! Baudrate {baudrate} empfängt 'U' Zeichen!")
                    ser.close()
                    return True
        
        # Verbindung schließen
        ser.close()
        
        # Zusammenfassung der empfangenen Daten
        if received_data:
            all_data = b''.join(received_data)
            print(f"  Zusammenfassung: {all_data}")
            print(f"  Als Text: {all_data.decode('ascii', errors='replace')}")
            
            # Prüfe auf lesbare ASCII-Zeichen
            readable_chars = sum(32 <= b <= 126 for b in all_data)
            total_chars = len(all_data)
            if total_chars > 0:
                readable_ratio = readable_chars / total_chars
                print(f"  Lesbare Zeichen: {readable_ratio*100:.1f}%")
                
                if readable_ratio > 0.7:  # Wenn mehr als 70% lesbar sind
                    print(f"\nMÖGLICHER TREFFER: Baudrate {baudrate} liefert lesbare Zeichen!")
                    return True
        
        return False
    
    except Exception as e:
        print(f"  Fehler bei Baudrate {baudrate}: {e}")
        return False

def main():
    print(f"Baudratentester für {PORT}")
    print("Suche nach 'U' Zeichen und lesbaren Daten...\n")
    
    success_baudrates = []
    
    # Teste alle Baudraten
    for baudrate in BAUDRATES:
        if test_baudrate(PORT, baudrate):
            success_baudrates.append(baudrate)
        print("")  # Leerzeile zur besseren Lesbarkeit
    
    # Ergebniszusammenfassung
    if success_baudrates:
        print("\nErfolgreiche Baudraten gefunden:")
        for baudrate in success_baudrates:
            print(f"- {baudrate} bps")
    else:
        print("\nKeine passende Baudrate gefunden.")
        print("Versuche weitere Baudraten oder überprüfe die Verbindung.")

if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print("\nTest durch Benutzer abgebrochen.")
        sys.exit(0)
